package model;

/**
 * Interface to Dots game model.
 * 
 * @author dennis
 *
 */
public interface DotsModelInterface {

	/**
	 * The possible team colours. Every game involes a white and a black team.
	 */
	public enum TeamColour { WHITE, BLACK, NONE;

		public TeamColour next() {
			return (this == WHITE) ? BLACK : WHITE;
		}
	}
		
	/** Set size of the game board. 
	 * Note that number of horizontal segments = cols-1 and
	 * number of vertical segments = rows-1
	 *  
	 * @param rows number of rows of dots
	 * @param cols number of columns of dots
	 */
	public void setSize(int rows, int cols);
	
	/** Reset game to initial state.
	 */
	public void reset();
	
	/** Set the observer for this model. 
	 * @param o Observer that should be notified via o.refresh() 
	 *           with any change to model.
	 */
	public void setObserver(Observer o);
	
	/** Place a horizontal segment on the board.
	 * @param r row to add in (0 = top)
	 * @param c column to add to right of (0 = leftmost)
	 * @return true if segment added successfully, false otherwise
	 */
	public boolean addHorizontalSegment(int r, int c);
	
	/** Place a vertical segment on the board.
	 * @param r row to add below (0 = top)
	 * @param c column to add in (0 = leftmost)
	 * @return true if segment added successfully, false otherwise
	 */
	public boolean addVerticalSegment(int r, int c);
	
	/**
	 * @return number of rows for game
	 */
	public int getRows();
	
	/**
	 * @return number of columns for game
	 */
	public int getCols();
	
	/** Get the colour of the horizontal segment.  
	 * @param r row containing the segment.
	 * @param c column to left of segment.
	 * @return colour of the segment or None if empty.
	 */
	public TeamColour getHorizontalSegmentColour(int r, int c);
	
	/** Get the colour of the vertical segment.  
	 * @param r row above the segment.
	 * @param c column containing the segment.
	 * @return colour of the segment or None if empty.
	 */
	public TeamColour getVerticalSegmentColour(int r, int c);
	
	/** Get the current score for a player.
	 * @param t player colour to get score for (White or BLACK)
	 * @return the score for that player
	 */
	public int getScore(TeamColour t);
	
	/** Get the player colour who's turn it is now.
	 * @return colour of team who gets to add next segment.
	 */
	public TeamColour getCurrentTurn();
	
	/** Check if game is over.
	 * @return true if more segments can be added to this game, false otherwise
	 */
	public boolean canPlayContinue();
	
	/** Default number of rows on board. */
	final static int defaultRows = 8;
	
	/** Default number of columns on board. */
	final static int defaultCols = 8;

}
