/**
 * BoardView.java
 * @author dennis
 * Created: Jan 17, 2006
 */
package view;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import model.DotsModelInterface;

/**
 * View for the playing board in Dots game.
 * @author dennis
 *
 */
class BoardView extends JPanel {

	/**
	 * 
	 */
	public BoardView(DotsModelInterface game) {
		mGame = game;
	    setPreferredSize(new Dimension((mGame.getCols()+1)*MIN_SPACING, (mGame.getRows()+1)*MIN_SPACING));
	    mSpacing = MIN_SPACING;
	    addComponentListener(new boardComponentListener());

	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g;
	    g2.setStroke(new BasicStroke(BORDER_WIDTH));
	    g.drawRect(1,1, getWidth()-BORDER_WIDTH, getHeight()-BORDER_WIDTH);
	
		drawDots(g2);
		drawSegments(g2);	
	}
	
	/** Find the game row or column containing p. 
	 *  If p is a y coordinate, then checks for a row, if p is an x coordinate
	 *  then checks for a column. The calculation is the same, so caller just has
	 *  to know what is represented by p.
	 * @param p Board coordinate (x or y) to check.
	 * @return row or col # if p is within +/- SEGMENT_WIDTH/2 of a row/col, -1 otherwise.
	 */
	public int getRowOrCol(int p) {
		int result = -1;
		// adjust for offsets & ensure p_offset is > base
		int p_offset = p+SEGMENT_WIDTH/2-mSpacing/2-BORDER_WIDTH;
		if (p_offset >= 0 && p_offset % mSpacing <= SEGMENT_WIDTH) {
			result = p_offset/mSpacing;
		}
		return result;
	}
		
	/** Find the row / column of the vertical / horizontal segment containing p.
	 * Similar to getRowOrCol except that this one looks along the long part of a
	 * segment.
	 * @param p Board coordinate x component
	 * @return column # to the left of x if x is on a segment, -1 otherwise
	 */
	public int getRowOrColBetween(int p) {
		int result = -1;
		int p_offset = p-mSpacing/2-BORDER_WIDTH-(DOT_SIZE+1);
		if (p_offset >= 0 && p_offset % mSpacing <= mSpacing - 2*(DOT_SIZE+1)) {
			result = p_offset / mSpacing;
		}
		return result;
	}
	
	private void drawDots(Graphics2D g) {
		int rows = mGame.getRows();
		int cols = mGame.getCols();
		g.setColor(Color.BLACK);
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				g.fillOval(graphicsX(c)-DOT_SIZE/2, graphicsY(r)-DOT_SIZE/2, DOT_SIZE, DOT_SIZE);
			}
		}
	}
	
	/** Draw the segments on the board
	 * @param g graphics context to draw on
	 */
	private void drawSegments(Graphics2D g) {
		int rows = mGame.getRows();
		int cols = mGame.getCols();
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols-1; c++) {
				drawHorizontalSegment(g, r, c);
			}
		}
		for (int r = 0; r < rows-1; r++) {
			for (int c = 0; c < cols; c++) {
				drawVerticalSegment(g, r, c);
			}
		}
	
	}
	
	/** Draw a horizontal segment at (r, c) if required.
	 * @param g graphical context
	 * @param r row
	 * @param c column
	 */
	private void drawHorizontalSegment(Graphics2D g, int r, int c) {
		DotsModelInterface.TeamColour sColour = mGame.getHorizontalSegmentColour(r, c);
		if (sColour != DotsModelInterface.TeamColour.NONE) {
			g.setColor(segmentColour(sColour));
			int tlx = graphicsX(c)+DOT_SIZE+1;
			int tly = graphicsY(r)-SEGMENT_WIDTH/2;
			g.fillRect(tlx, tly, mSpacing - 2*(DOT_SIZE+1), SEGMENT_WIDTH);
		}
	
	}

	/** Draw a vertical segment at (r, c) if required.
	 * @param g graphical context
	 * @param r row
	 * @param c column
	 */
	private void drawVerticalSegment(Graphics2D g, int r, int c) {
		DotsModelInterface.TeamColour sColour = mGame.getVerticalSegmentColour(r, c);
		if (sColour != DotsModelInterface.TeamColour.NONE) {
			g.setColor(segmentColour(sColour));
			int tlx = graphicsX(c)-SEGMENT_WIDTH/2;
			int tly = graphicsY(r)+DOT_SIZE+1;
			g.fillRect(tlx, tly, SEGMENT_WIDTH, mSpacing - 2*(DOT_SIZE+1));
		}
	
	}

	private Color segmentColour(DotsModelInterface.TeamColour t) {
		return (t == DotsModelInterface.TeamColour.WHITE ? Color.WHITE : Color.BLACK);
	}
	
	/** Adjust the scale for the new window size.
	 */
	private void rescale() {
		double xOptimum = (double)(getWidth()-2*BORDER_WIDTH)/(double)(mGame.getCols());
		double yOptimum = (double)(getHeight()-2*BORDER_WIDTH)/(double)(mGame.getRows());
		
		mSpacing = (int)((xOptimum < yOptimum) ? xOptimum : yOptimum);
		mSpacing = (mSpacing < MIN_SPACING) ? MIN_SPACING : mSpacing;
//		System.out.println("Spacing = " + mSpacing);
	}
	
	/** Compute the graphics x corresponding to the center of a given column.
	 * Assumes that mSpacing is updated.
	 * @param c a column number (0 is leftmost)
	 */
	private int graphicsX(int c) {
		return (int)(c*mSpacing+mSpacing/2+BORDER_WIDTH);
	}
	
	/** Compute the graphics y corresponding to the center of a given row.
	 * Assumes that mSpacing is updated.
	 * @param r a row number (0 is top)
	 */
	private int graphicsY(int r) {
		return (int)(r*mSpacing+mSpacing/2+BORDER_WIDTH);
	}
	
	/**  The game model. */
	private DotsModelInterface mGame;
	
	/** Minimum distance between dots */
	private static final int MIN_SPACING = 30;
	
	private static final int BORDER_WIDTH = 2;
	
	/** Spacing between dots */
	private int mSpacing;
	
	/** Diameter of dots */
	private static final int DOT_SIZE = 8;
	
	/** Width of the segments. */
	private static final int SEGMENT_WIDTH = 12;
	
	/** Catch resize of the BoardView to adjust scale. 
	 */
	private class boardComponentListener 
	extends ComponentAdapter
	{
		public void componentResized(ComponentEvent e) {
			rescale();
		}
	}

}
